/*
 * @(#) ApplicationFrame.java 1.0 98/20/5
 * Copyright (c) 1999 Larry Rodrigues
 */
  import java.awt.*;
  import java.awt.event.*;
  import java.util.*;
  import java.io.*;
  import javax.swing.*;
  import javax.swing.event.*;
  import java.awt.print.*;
  /** Provides a skeleton application frame for applications.
    * @version 1.0  26 dec 1999
    * @author Lawrence Rodrigues
    */
public class ApplicationFrame extends JFrame{
   private int width, height;
   String str;
   Locale curLocale;
   String copyRight, author;
   String helpMidText, helpBoxTitle;
   String aboutString, aboutTitle;
   JMenuItem open, exit,save, saveas, print, help, about;
   JMenu lookAndFeel;
   protected String curDataFile;
   protected Container contentPane;
   protected Printable printerObj =null;
   public String saveFileName;

   public static void main(String arg[]){
      ApplicationFrame app = new ApplicationFrame(400, 300);
      app.show();
   }

   public ApplicationFrame(){
      contentPane = this.getContentPane();
      setLocale(Locale.US);
      createGUI();
   }

   /**  Constructor when used in applications.
     *  @param wid the width of the frame.
     *  @param ht the height of the frame.
     */
   public ApplicationFrame(int wid, int ht){
      setTitle("");
      width = wid;
      height = ht;
      createGUI();
      setSize(width,height);
   }

   protected void createGUI(){
      setBackground(Color.lightGray);
      WindowEventHandler winAda = new WindowEventHandler();
      addWindowListener(winAda);
      AppMenuBar AppMenuBar = new AppMenuBar();
      setJMenuBar((JMenuBar)AppMenuBar);
   }
   public void setPrinterObj(Printable pr){
      printerObj = pr;
   }

   public void setAboutString(String str){
      aboutString = str;
   }

    public void setAboutTitle(String str){
      aboutTitle = str;
   }

   public void updateUI(){
      SwingUtilities.updateComponentTreeUI(contentPane);
   }

   class WindowEventHandler extends WindowAdapter{
       public void windowClosed(WindowEvent e){

       }
       public void windowClosing(WindowEvent e){
          System.exit(0);
       }
   }

   protected void open(){

   }

   protected void saveAs(){
   }

   protected void save(){

   }

   protected void printContents() {

   }

   public void quit(){
      dispose();
      System.exit(0);
   }

   public class AppMenuBar extends JMenuBar {
      Vector menus = new Vector();
      public AppMenuBar(){
         createGUI();
      }

      public void createGUI(){
         createFileMenu();
      }

      public JMenu createFileMenu() {
         JMenu fileMenu = new JMenu("File");
         exit = new JMenuItem("Exit");
         exit.addActionListener(
           new ActionListener(){
              public void actionPerformed(ActionEvent e){
                 quit();
             }
           }
         );
         fileMenu.add(createSaveMenu());
         fileMenu.add(createPrintMenu());
         fileMenu.addSeparator();
         fileMenu.add(exit);
         add(fileMenu);
         return fileMenu;
     }
      public JMenu createSaveMenu() {
         JMenuItem save = new JMenuItem("Save");
         save.addActionListener(
            new ActionListener() {
               public void actionPerformed(ActionEvent e){
                 save();
               }
            }
         );
         JMenuItem saveas = new JMenuItem("Save As");

         saveas.addActionListener(
            new ActionListener() {
               public void actionPerformed(ActionEvent e){
                 saveAs();
               }
            }
         );

         JMenu saveMenu = new JMenu("Save");
         saveMenu.add(save);
         saveMenu.add(saveas);
         add(saveMenu);
         return saveMenu;
      }

      public JMenu createHelpMenu() {
         help = new JMenuItem("Help");
         help.addActionListener(
            new ActionListener() {
               public void actionPerformed(ActionEvent e){
                  JOptionPane.showMessageDialog(ApplicationFrame.this,
                  " Under Construction",
                  "Help",
                  JOptionPane.INFORMATION_MESSAGE);
               }
            }
         );

         about = new JMenuItem("About");
         about.addActionListener(
            new ActionListener() {
               public void actionPerformed(ActionEvent e){
                  JOptionPane.showMessageDialog(ApplicationFrame.this,
                                                aboutString,
                                                aboutTitle,
                                                JOptionPane.INFORMATION_MESSAGE);
               }
            }
         );
         JMenu helpMenu = new JMenu("Help");
         helpMenu.add(help);
         helpMenu.add(about);
         add(helpMenu);
         return helpMenu;
      }

      public JMenu createPrintMenu(){
         print = new JMenuItem("Print");
         PrintAdapter pa = new PrintAdapter(this);
         print.addActionListener(pa);
         JMenuItem printSetup = new JMenuItem("Print Setup");
         JMenu printMenu = new JMenu("Print");
         printMenu.add(print);
         menus.addElement(printMenu);
         add(printMenu);
         return printMenu;
      }

      public void addMenus(Vector menus){
         for(Enumeration e = menus.elements();e.hasMoreElements();){
             add((JMenu)(e.nextElement()));
         }
      }
      class PrintSetupAdapter implements ActionListener{
         public void actionPerformed(ActionEvent e){
            PrintJob pr =
            Toolkit.getDefaultToolkit().getPrintJob(ApplicationFrame.this, "Print", null);
            Graphics printG = pr.getGraphics();
            printAll(printG);
            printG.dispose();
            pr.end();
         }

     }

     class PrintAdapter implements ActionListener{
         public PrintAdapter(AppMenuBar mb){
            //mb.super();
         }
         public void actionPerformed(ActionEvent e){
            //JDK 1.2 print
            printContents();
        /*
            if(printerObj == null) return;
            PrinterJob pj = PrinterJob.getPrinterJob();
            PageFormat pm = pj.pageDialog(pj.defaultPage());
            pj.setPrintable(printerObj);
            try{
                pj.print();
            }catch(Exception e1){}
        */
         }
     }

     class LocaleAdapter implements ActionListener{
         Locale locale;
         public LocaleAdapter(Locale loc){
            locale = loc;
         }
         public void actionPerformed(ActionEvent e){
            ApplicationFrame.this.setLocale(locale);
         }
      }
      class OpenAdapter implements ActionListener{
          public void actionPerformed(ActionEvent e){
             open();
          }
       }


       class SaveAdapter implements ActionListener{
          public void actionPerformed(ActionEvent e){
             save();
          }
       }

       class SaveAsAdapter implements ActionListener{
          public void actionPerformed(ActionEvent e){
             saveAs();
          }
       }

    }  //End of AppMenuBar class
}
